using System;
using Rational.Test.Ft.Value.Managers;

namespace SDK.Sample.Value
{
	/// <summary>
	/// Summary description for SimpleValueManager.
	/// </summary>
	public class SimpleValueManager: IManageValueClass
	{
		private const System.String CLASSNAME = "SDK.Sample.Value.SimpleValue";
		private const System.String CANONICALNAME = ".simpe_value";

		private const System.String DATA = "Data";

		///<summary>
		/// This method pushes the significant data associated with an object
		/// into the specified output object.  Note that the data persisted 
		/// must be either a primitive type or have a <see cref="IManageValueClass"/>IManageValueClass in the value class map to support the data class.
		/// <br/><br/>
		/// Supplied is an <see cref="IAuxiliaryDataManager"/>IAuxiliaryDataManager instance that
		/// should be used to name auxiliary data files.  Failure to use this
		/// object to name auxiliary data will result in the auxiliary files 
		/// not be properly managed by the test management system.</summary>
		///<param name="theObject">The object that is to be sequentially inserted 
		///						into the supplied output stream.</param>
		///<param name="persist">The output stream to sequentially insert data
		///						into.</param>
		///<param name="auxData">The auxiliary data interface used for naming
		///						associated data files.</param>
		///<see cref="PersistIn(IPersistIn,IAuxiliaryDataManager)"/>
		///<see cref="PersistIn(IPersistInNamed,IAuxiliaryDataManager)"/>
		///<see cref="IPersistOut"/>
		///<see cref="IAuxiliaryDataManager"/>
		public virtual void  PersistOut(System.Object theObject, IPersistOut persist, IAuxiliaryDataManager auxData)
		{
			SimpleValue simple = (SimpleValue)theObject;
			persist.Write(DATA, simple.GetValue());
		}
		
		///<summary>
		/// This method pulls data from a supplied input object.  The
		/// data must be pulled in the same order in which it was pushed by
		/// <see cref="PersistOut(Object,IPersistOut,IAuxiliaryDataManager)"/>persistOut.
		/// <br/><br/>
		/// Supplied is an <see cref="IAuxiliaryDataManager"/>IAuxiliaryDataManager 
		/// instance that should be used to locate auxiliary data files.  
		/// Auxiliary data files that do not follow the supplied naming 
		/// convention will not be managed  by the test management system.</summary>
		///<param name="persist">The input stream that data is pulled sequentially
		///						out of.</param>
		///<param name="auxData">The auxiliary data interface used for naming
		///						associated data files.</param>
		///<returns>The object extracted from the data stream.</returns>
		///<see cref="PersistOut(Object,IPersistOut,IAuxiliaryDataManager)"/>
		public virtual System.Object PersistIn(IPersistIn persist, IAuxiliaryDataManager auxData)
		{
			String data = (String)persist.Read(0);
			return new SimpleValue(data);
		}
		
		///<summary>
		/// This method pulls data from a supplied input object.  The
		/// data must be pulled in using the names supplied when the 
		/// object was persisted by
		/// <see cref="PersistOut(Object,IPersistOut,IAuxiliaryDataManager)"/>persistOut.
		/// <br/><br/>
		/// Supplied is an <see cref="IAuxiliaryDataManager"/>IAuxiliaryDataManager 
		/// instance that should be used to locate auxiliary data files.  
		/// Auxiliary data files that do not follow the supplied naming 
		/// convention will not be managed by the test management system.</summary>
		///<param name="persist">The input object that the named data is pulled 
		///						out of.</param>
		///<param name="auxData">The auxiliary data interface used for naming
		///						associated data files.</param>
		///<returns>The object extracted from the data stream.</returns>
		///<see cref="PersistOut(Object,IPersistOut,IAuxiliaryDataManager)"/>
		public virtual System.Object PersistIn(IPersistInNamed persist, IAuxiliaryDataManager auxData)
		{
			String data = (String)persist.Read(DATA);
			return new SimpleValue(data);
		}
		
		///<summary>
		/// This method compares objects supported by this value class manager
		/// and returns a result in the range <b>0</b> to <b>100</b>.  If the
		/// objects are equivalent this method will return <b>100</b>.  If the
		/// objects have no similarities <b>0</b> will be returned.  If the
		/// objects are partially equivalent then an appropriate value between
		/// <b>0</b> and <b>100</b> will be returned.  Commonly only <b>0</b> 
		/// and <b>100</b> are returned by an Implementation(based on straight
		/// equality "<code>((left.Equals(right)) ? 100 : 0 )</code>").
		/// <br/><br/>
		/// An example of when a value other then <b>0</b> or <b>100</b> will 
		/// be returned is a <i>caption</i>.  Window captions commonly contain 
		/// dynamic information such as file name or date type information.  A 
		/// partial match on a caption might return a <b>50</b> rather then a
		/// <b>0</b> or <b>100</b>.</summary>
		///<param name="left">The first object in the comparison.</param>
		///<param name="right">The second object in the comparison.</param>
		///<param name="nested">An object that can perform nested complex 
		///						<i>compare</i> actions.</param>
		///<returns>An integer value in the range <b>0</b> to <b>100</b> that
		///			depicts the level of equality.  <b>0</b> is poor and 
		///			<b>100</b> is good.</returns>
		public virtual int Compare(System.Object left, System.Object right, ICompareValueClass nested)
		{
			if ( left == null || right == null )
				return ( left == right ? 100 : 0 );
			if ( !(right is SimpleValue) )	return 0;

			SimpleValue l = (SimpleValue)left;
			SimpleValue r = (SimpleValue)right;
			return ( l.Equals(r) ? 100 : 0 );
		}
		
		
		///<summary>
		/// This method is essentially a copy constructor for an object of the managed
		/// class.  The copy does not have to be a deep copy and should return <code>
		/// null</code> if a copy operation is not be performed.  The primary use of this
		/// method is to shear property methods returning extended instances of the
		/// expected class.  For instance if <code>java.awt.Point</code> is the expected
		/// return type, however <code>mycompany.MyPoint</code> (which extends the AWT 
		/// Point class) is actually returned, then the returned object will need to be
		/// sheared before it may be used as a Point property.  To get the <code>MyPoint
		/// </code> object use an <see cref="IManageValueClass"/>IManageValueClass will need
		/// to be crated for the new Class(it must be a <i>value class</i> object).</summary>
		///<param name="sourceToCopy">An instance of the associated value class.</param>
		///<returns>A pure copy of the expected value class.</returns>
		public virtual System.Object CreateValue(System.Object sourceToCopy)
		{
			if ( sourceToCopy is SimpleValue )
				return new SimpleValue(((SimpleValue)sourceToCopy).GetValue());
			return null;
		}
		
		///<summary>
		/// This method returns a canonical name for the value class that uniquely identifies
		/// the streamed format of the data associated with this class.  Verification
		/// point data may be read on more then one platform via multiple languages.
		/// The data format remains the same, however the associated <see cref="IManageValueClass"/>IManageValueClass may vary from platform to platform.  This method 
		/// returns a name that does not vary between platforms.  
		/// <br/><br/>
		/// For instance
		/// <i>rational.hashtable</i> may be returned for the set of key/value pairs
		/// associated with a <code>java.util.hashtable</code>.  On another platform
		/// this data may be read into a C++ STL grid type object.  Regardless the
		/// data can be read because a canonical name for the value class has been associated
		/// with the data.  Note that the name in this example uses the <code>rational
		/// </code> company name so that this value class name for a hashtable does
		/// not conflict with another companies.
		/// <br/><br/>
		/// This name is automatically added to the data stream for an <see cref="IManageValueClass"/>IManageValueClass mapped object and therefore does not need to be 
		/// explicitly added by an implementation of <see cref="PersistOut(Object,IPersistOut,IAuxiliaryDataManager)"/>persistOut.</summary>
		///<returns>The canonical name for the value class.</returns>
		public virtual System.String GetCanonicalName()
		{
			return CANONICALNAME;
		}
		
		
		///<summary>
		/// This method returns the class name of the object that this manager 
		/// represents.  Each <see cref="IManageValueClass"/>IManageValueClass can only support one value
		/// class.  The name of that class is returned by this method so that the
		/// mapping of the value class to the appropriate value class manager can
		/// be performed.</summary>
		///<returns>The full class name of the associated value class.</returns>
		public virtual System.String GetClassName()
		{
			return CLASSNAME;
		}
	}
}
